#define STRICT
#include <windows.h>
#pragma hdrstop

#include "rc.h"
#include "lib\dsc_connect.h"

#define version "v0.03"
#define compilation_time __DATE__ " " __TIME__ 

const char* ProgName = "Coolpix RAW Enabler" " " version;
DSC* dsc;
HINSTANCE hAppInst;
HWND hAppWin;

bool raw_enabled = false;
bool changed = false;

void InitDlg(HWND hDlg)
{
   SetDlgItemText(hDlg, IDC_MESSAGE, "Connecting to camera...");
   EnableWindow( GetDlgItem(hDlg, IDC_RAW_ENABLED), FALSE );
   EnableWindow( GetDlgItem(hDlg, IDOK), FALSE );

   PostMessage(hDlg, WM_COMMAND, IDC_CONNECT, 0);
}

bool GetRAWMode(HWND hDlg)
{
   if( !dsc ) return false;
   
   TResponce responce;

   if( dsc->GetVar(0x16, responce) )
   {
      raw_enabled = false;
      if( strstr(responce.GetStr(), "DIAG") != 0 && strstr(responce.GetStr(), "RAW") != 0 )
         raw_enabled = true;

      EnableWindow( GetDlgItem(hDlg, IDC_RAW_ENABLED), TRUE );

      CheckDlgButton(hDlg, IDC_RAW_ENABLED, raw_enabled ? BST_CHECKED:BST_UNCHECKED);
      return true;
   }
   return false;
}

enum OSV { OSV_NONE, OSV_USB_IOCTL = 1, OSV_USB_ASPI = 2, OSV_COM = 4 };

int GetOSVersion()
{
   OSVERSIONINFO osv;
   osv.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
   if( !GetVersionEx( &osv ) ) return OSV_NONE;

   DOUT(("GetVersionEx: %u v%u.%02u.%04u (%s)\n", osv.dwPlatformId, osv.dwMajorVersion, osv.dwMinorVersion, osv.dwBuildNumber, osv.szCSDVersion));

   if( osv.dwMajorVersion == 4 )
      return OSV_USB_ASPI|OSV_COM;
      
   if( osv.dwMajorVersion >= 5 )
      return OSV_USB_IOCTL|OSV_COM|OSV_USB_ASPI;
      
   return OSV_COM;
}

bool Connect(HWND hDlg)
{
   TDeviceInfo info;
   char mess[80];

   int osv = GetOSVersion();
   if( !dsc && (osv & OSV_USB_IOCTL) )
   {
      dsc = OpenUSB();
   }
#ifdef SUPPORT_WINDOWS_98SE
   if( !dsc && (osv & OSV_USB_ASPI) )
   {
      dsc = OpenUSB_ASPI();
   }
#endif
#ifdef SUPPORT_SERIAL_COM
   if( !dsc && (osv & OSV_COM) )
   {
      dsc = OpenCOM();
   }
#endif

   if( !dsc )
      return false;

   if( dsc->GetDeviceInfo( info ) )
   {
      wsprintf(mess, "%s %s%s", dsc->comment, info.ProductId, info.ProductRevisionLevel);
   }
   else
   {
      wsprintf(mess, "Camera connected via %s", dsc->comment );
   }

   SetDlgItemText(hDlg, IDC_MESSAGE, mess);
   return true;
}

bool Update()
{
   if( !dsc ) return false;
   return dsc->SetStr(0x16, raw_enabled ? "DIAG=RAW" : "NIKON DIGITAL CAMERA" );
}

BOOL Command(HWND hDlg, int NotifyCode, int id, HWND hCtrl)
{
   NotifyCode;
   hCtrl;

   switch(id)
   {
   case IDCANCEL:
      EndDialog(hDlg, 0);
      return TRUE;

   case IDOK:
      // Update
      if( changed )
      {
         if( Update() )
            MessageBox(hDlg, raw_enabled ?
               "RAW mode enabled":"RAW mode disabled", ProgName, MB_ICONINFORMATION|MB_OK);
         else
            MessageBox(hDlg, "Cannot change RAW mode", ProgName, MB_ICONSTOP|MB_OK);
      }
      EndDialog(hDlg, 1);
      return TRUE;

   case IDC_RAW_ENABLED:

      if( !changed )
         EnableWindow( GetDlgItem(hDlg, IDOK), TRUE );

      changed = true;
      raw_enabled = ! raw_enabled;

      CheckDlgButton(hDlg, IDC_RAW_ENABLED, raw_enabled ? BST_CHECKED:BST_UNCHECKED);
      return TRUE;

   case IDC_CONNECT:

      if( !Connect(hDlg) )
      {
         SetDlgItemText(hDlg, IDC_MESSAGE, "Camera is not found");

         MessageBox(hDlg,
            "Can not connect to camera.\r\n"
            "Check USB or COM connection \r\n"
            "and try again.", ProgName, MB_OK|MB_ICONSTOP);

         EndDialog(hDlg, 0);
         return TRUE;
      }

      if( !GetRAWMode(hDlg) )
         GetRAWMode(hDlg);
         
      return TRUE;

   default:
      return FALSE;
   }
}

BOOL CALLBACK MainDlg( HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   if( uMsg == WM_INITDIALOG )
   {
      if( !hAppWin )
         hAppWin = hDlg;

      InitDlg(hDlg);
      return TRUE;
   }

   if( uMsg == WM_COMMAND )
   {
      return Command(hDlg, HIWORD(wParam), LOWORD(wParam), (HWND) lParam);
   }

   if( uMsg == WM_CLOSE )
   {
      EndDialog(hDlg, 0);
      return TRUE;
   }

   if( uMsg == WM_DESTROY )
   {
      if( hDlg == hAppWin )
         hAppWin = NULL;

      return FALSE;
   }
   return FALSE;
}

int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, LPSTR, int )
{
   hAppInst = hInstance;

#ifdef DEBUG
   open_log("cpixraw.log");
#endif

   DOUT(("Run: cpixraw " version " [" compilation_time "]\n"))

   DialogBox( hAppInst, MAKEINTRESOURCE(IDD_MAIN), NULL, MainDlg);

   delete dsc;
   dsc = NULL;

   DOUT(("Normal exit\n"))

#ifdef DEBUG
   close_log();
#endif
   return 0;
}

